//
//  ScanController.swift
//  iOS_ble_app
//
//  Created by Nicolas Wiedmer on 03.09.18.
//  Copyright © 2018 Nicolas Wiedmer. All rights reserved.
//

import UIKit
import CoreBluetooth

class ScanController: UIViewController {

    private let dataSource = TableViewDataModel()
    private var count = 1
    var bluetoothManager : BluetoothManager!
    private var peripherals: [CBPeripheral]!
    
    var duration : Int = 20
    var timer = Timer()
    var isTimerRunning: Bool = false
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(true)
        
        dataSource.requestData()
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        print("ScanController View did disappear")
    }
    
    
    fileprivate var dataArray = [TableViewDataModelItem]() {
        didSet {
            tableView?.reloadData()
        }
    }
    
    func runTimer() {
        timer = Timer.scheduledTimer(timeInterval: 1, target: self, selector: #selector(updateTimer), userInfo: nil, repeats: true)
        isTimerRunning = true
    }
    
    func stopAll() {
        timer.invalidate()
        bluetoothManager.stopScan()
        isTimerRunning = false
        duration = 10
    }
    
    @objc func updateTimer() {
        if duration < 1 {
            stopAll()
        } else {
            self.duration -= 1
        }
    }
    
    @IBOutlet weak var tableView: UITableView!
    @IBAction func btnDiscover(_ sender: UIButton) {
        if isTimerRunning == false {
            dataArray.removeAll()
            bluetoothManager.discoverDevice()
            runTimer()
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        print("ScanController did load")
        
        NotificationCenter.default.addObserver(self, selector: #selector(bluetoothOnNotification), name: RCNotification.BluetoothReady, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(managerStartScanningNotification), name: RCNotification.StartScanning, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(managerStopsScanningNotification), name: RCNotification.StopScanning, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(scanningFoundDeviceNotification(notification:)), name: RCNotification.FoundDevice, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(disconnectDeviceNotification), name: RCNotification.DisconnectedDevice, object: nil)
        
        tableView?.register(TableViewCell.nib, forCellReuseIdentifier: TableViewCell.identifier)
        tableView?.delegate = self
        tableView?.dataSource = self
        
        dataSource.delegate = self
        
        bluetoothManager = BluetoothManager.sharedInstance()
        bluetoothManager.initCentralManager()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(true)
        print("Scan View did appear")
    }
    
    @objc func bluetoothOnNotification() {
        print("Bluetooth On with Notification sent")
    }
    
    @objc func managerStartScanningNotification() {
        print("manager starts scanning Notification sent")
    }
    
    @objc func managerStopsScanningNotification() {
        print("manager stops scanning Notification sent")
    }
    
    @objc func scanningFoundDeviceNotification(notification: NSNotification) {
        let description: [AnyHashable: Any] = notification.userInfo!
        let data : [String: String] = [
            "deviceName": description[AnyHashable("deviceName")] as! String,
            "deviceAddress": description[AnyHashable("deviceAddress")] as! String,
            "deviceRSSI": description[AnyHashable("deviceRSSI")] as! String
        ]
        let newDevice = TableViewDataModelItem(data: data)!
        dataArray.append(newDevice)
        peripherals = bluetoothManager.getPeripherals()
    }
    
    @objc func disconnectDeviceNotification() {
        print("Device was disconnected Notification sent")
    }
}

extension ScanController: UITableViewDelegate {
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        print("Cell selected at \(indexPath.row)")
        let selectedCell: TableViewCell = tableView.cellForRow(at: indexPath) as! TableViewCell
        print(selectedCell.getTableViewDataModelItem().deviceName! + " " + selectedCell.getTableViewDataModelItem().deviceAddress!)
        if let mvc = UIStoryboard(name: "Main", bundle: nil).instantiateViewController(withIdentifier: "MainController") as? MainController {
            print("Switch to mainController...")
            stopAll()
            let device: CBPeripheral = peripherals[indexPath.row]
            bluetoothManager.connectToDevice(device: device)
            mvc.setBluetoothManager(manager: bluetoothManager)
            mvc.setDeviceDescription(item: selectedCell.getTableViewDataModelItem())
            self.present(mvc, animated: true, completion: nil)
        } else {
            print("Could not instanciate MainController")
        }
    }
    
    func tableView(_ tableView: UITableView, didDeselectRowAt indexPath: IndexPath) {
        print("Cell deselected at \(indexPath.row)")
        let cellToDeselect:UITableViewCell = tableView.cellForRow(at: indexPath)!
        cellToDeselect.contentView.backgroundColor = .clear
    }
}

extension ScanController: UITableViewDataSource {
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if let cell = tableView.dequeueReusableCell(withIdentifier: TableViewCell.identifier, for: indexPath) as? TableViewCell {
            cell.configureWithItem(item: dataArray[indexPath.item])
            return cell
        }
        return UITableViewCell()
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return dataArray.count
    }
}

extension ScanController: TableViewDataModelDelegate {
    func didReceiveDataUpdate(data: [TableViewDataModelItem]) {
        dataArray = data
    }
    
    func didFailDataUpdateWithError(error: Error) {
        print("error: \(error.localizedDescription)")
    }
    
}










