//
//  SettingsViewController.swift
//  iOS_ble_app
//
//  Created by Nicolas Wiedmer on 11.10.18.
//  Copyright © 2018 Nicolas Wiedmer. All rights reserved.
//

import UIKit
import CoreBluetooth

class SettingsViewController: UIViewController {

    @IBOutlet weak var collectionView: UICollectionView!
    @IBOutlet weak var timeLabelsStackView: UIStackView!
    @IBOutlet weak var scrollViewTimes: UIScrollView!
    @IBOutlet weak var collectionViewFlowLayout: UICollectionViewFlowLayout!
    @IBOutlet weak var stackViewBottomConstraint: NSLayoutConstraint!
    @IBOutlet weak var stackViewTopConstraint: NSLayoutConstraint!
    
    private let dataSource = CollectionViewDataModel()
    private var saveSettingsText: String = ""
    private let file = "SavedSettings.txt"
    private var fileURL: URL? = nil
    private var didSettingsChange = false
    private var dimensions: CGFloat!
    private var tsService: TimeSettingsService!
    private var isLayoutSet : Bool = false
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(true)
        
        dataSource.requestData()
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        tsService = nil
        print("Settings Controller View did disappear")
    }
    
    fileprivate var dataArray = [CollectionViewDataModelItem]() {
        didSet {
            collectionView?.reloadData()
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        print("SettingsView did load")
        
        NotificationCenter.default.addObserver(self, selector: #selector(timeSettingsWriteCharFinishedNotification), name: RCNotification.TimeSettingsWriteCharFinished, object: nil)
        collectionView.register(LabelCollectionViewCell.nib, forCellWithReuseIdentifier: LabelCollectionViewCell.identifier)
        collectionView.dataSource = self
        collectionView.delegate = self
        scrollViewTimes.delegate = self
        
        collectionView.setCollectionViewLayout(collectionViewFlowLayout, animated: false)
        timeLabelsStackView.translatesAutoresizingMaskIntoConstraints = false
        
        dataSource.delegate = self
        
        collectionView.isScrollEnabled = false
        
        if let dir = FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first {
            fileURL = dir.appendingPathComponent(file)
        }
        
        loadSettings()
        
        // Do any additional setup after loading the view.
    }
    
    func loadSettings() {
        let text = readSettings()
        if text == "" {
            print("Settings are empty")
            for i in 0...167 {
                let data: [String: String] = [
                    "cellID": String(i),
                    "cellState": "ON"
                ]
                let newCell = CollectionViewDataModelItem(data: data)!
                dataArray.append(newCell)
            }
        } else {
            print("Settings are not empty and will initialized")
            let allSettings = text?.components(separatedBy: ";")
            for item in allSettings! {
                if item != "" {
                    var subSetting = item.components(separatedBy: ",")
                    let data: [String: String] = [
                        "cellID": subSetting[0],
                        "cellState": subSetting[1]
                    ]
                    let newCell = CollectionViewDataModelItem(data: data)!
                    dataArray.append(newCell)
                }
            }
        }
        
    }
    
    /*func setBluetoothManager(manager: BluetoothManager) {
        self.bluetoothManager = manager
    }*/
    
    func setTSService(service: TimeSettingsService) {
        tsService = service
    }
    
    @IBAction func btnSave(_ sender: UIButton) {
        saveSettingsText = ""
        for item in dataArray {
            saveSettingsText.append(item.cellID! + "," + item.cellState! + ";")
        }
        //print("Writing " + saveSettingsText)
        saveSettings(text: saveSettingsText)
        didSettingsChange = false
        tsService.setTimeSettings(settings: saveSettingsText)
    }
    
    @IBAction func btnCancel(_ sender: UIButton) {
        if(!didSettingsChange) {
            print("Switch back to Main Controller...")
            self.presentingViewController?.dismiss(animated: true, completion: nil)
        } else {
            let alert = UIAlertController(title: "Save Settings", message: "The changes aren't saved. Do you want to continue?", preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "Yes", style: .default, handler: {
                action in
                print("Switch back to Main Controller...")
                self.presentingViewController?.dismiss(animated: true, completion: nil)
            }))
            alert.addAction(UIAlertAction(title: "No", style: .cancel, handler: nil))
            
            self.present(alert, animated: true)
        }
    }
    
    @objc func timeSettingsWriteCharFinishedNotification() {
        print("Switch back to Main Controller...")
        self.presentingViewController?.dismiss(animated: true, completion: nil)
    }
    
    func saveSettings(text: String!) {
        //writing
        do {
            try text.write(to: fileURL!, atomically: false, encoding: .utf8)
        }
        catch {
            print("Error with writing Settings")
        }
    }
    
    func readSettings() -> String! {
        var text = ""
        //reading
        do {
            text = try String(contentsOf: fileURL!, encoding: .utf8)
            print("Reading: " + text)
        }
        catch {/* error handling here */
            print("Error with reading Settings")
        }
        return text
    }
    
    func setTimeLabelsSpacing() {
        if isLayoutSet {
            let spacFloat = (Float(collectionView.bounds.size.width) / Float(8))
            let spacing = spacFloat.truncatingRemainder(dividingBy: 1) * 8
            dimensions = CGFloat(Int(collectionView.bounds.size.width) / 8)
            
            if spacing >= 5 {
                collectionViewFlowLayout.minimumLineSpacing = CGFloat(spacing)
                timeLabelsStackView.spacing = CGFloat(spacing)
            } else {
                collectionViewFlowLayout.minimumLineSpacing = 5
                timeLabelsStackView.spacing = 5
            }
            
            let firstLabel : UIView = timeLabelsStackView.arrangedSubviews.first!

            let labelHeight = firstLabel.bounds.height
            let topSpacing = (dimensions - labelHeight) / 2
            let labelSpacing = topSpacing + CGFloat(spacing) + topSpacing
            stackViewTopConstraint.constant = topSpacing
            stackViewBottomConstraint.constant = topSpacing
            timeLabelsStackView.spacing = labelSpacing
        }
        
    }
}

extension SettingsViewController: UIScrollViewDelegate {
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        if scrollView == scrollViewTimes {
            self.synchronizeScrollView(collectionView, toScrollView: scrollViewTimes)
            scrollView.contentOffset.x = 0
        }
        /*else if scrollView == collectionView {
            self.synchronizeScrollView(scrollViewTimes, toScrollView: collectionView)
        }*/
    }
    
    func synchronizeScrollView(_ scrollViewToScroll: UIScrollView, toScrollView scrolledView: UIScrollView) {
        var offset = scrollViewToScroll.contentOffset
        offset.y = scrolledView.contentOffset.y
        
        scrollViewToScroll.setContentOffset(offset, animated: false)
    }
}

private var colView : UICollectionView!
extension SettingsViewController: UICollectionViewDelegateFlowLayout {
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        colView = collectionView
        return resizeColItems(width: colView.bounds.size.width)
    }
    
    private func resizeColItems(width: CGFloat) -> CGSize{
        let itemsPerRow: Int = 8
        dimensions = CGFloat(Int(width) / Int(itemsPerRow))
        if !isLayoutSet {
            isLayoutSet = true
            setTimeLabelsSpacing()
        }
        return CGSize(width: dimensions, height: dimensions)
    }
    
    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
        super.viewWillTransition(to: size, with: coordinator)
        isLayoutSet = false
        
    }
    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
        
        guard let flowLayout = collectionView.collectionViewLayout as? UICollectionViewFlowLayout else {
            return
        }
        
        flowLayout.invalidateLayout()
    }
}

extension SettingsViewController: UICollectionViewDelegate {
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        print("Cell selected at \(indexPath.row)")
        let selectedCell: LabelCollectionViewCell = collectionView.cellForItem(at: indexPath) as! LabelCollectionViewCell
        print(selectedCell.getCollectionViewDataModelItem().cellState! + " " + selectedCell.getCollectionViewDataModelItem().cellID!)
        if selectedCell.getCollectionViewDataModelItem().cellState == "ON" {
            selectedCell.getCollectionViewDataModelItem().setCellDescription(newCellState: "OFF")
        } else {
            selectedCell.getCollectionViewDataModelItem().setCellDescription(newCellState: "ON")
        }
        collectionView.reloadData()
        didSettingsChange = true
    }
}

extension SettingsViewController: UICollectionViewDataSource {
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        if let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "LabelCellIdentifier", for: indexPath) as? LabelCollectionViewCell {
            cell.configureWithItem(item: dataArray[indexPath.row])
            return cell
        }
        return UICollectionViewCell()
    }
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return dataArray.count
    }
}

extension SettingsViewController : CollectionViewDataModelDelegate {
    func didReceiveDataUpdate(data: [CollectionViewDataModelItem]) {
        dataArray = data
    }
    
    func didFailDataUpdateWithError(error: Error) {
        print("error: \(error.localizedDescription)")
    }
}
