//
//  BluetoothManager.swift
//  iOS_ble_app
//
//  Created by Nicolas Wiedmer on 04.09.18.
//  Copyright © 2018 Nicolas Wiedmer. All rights reserved.
//

import CoreBluetooth

class BluetoothManager: NSObject, CBCentralManagerDelegate, CBPeripheralDelegate {
    
    private var centralManager: CBCentralManager!
    private var bleDevice: CBPeripheral!
    private var psService: PowerSocketService!
    private var ctService: CurrentTimeService!
    private var tsService: TimeSettingsService!
    private var peripherals = [CBPeripheral]()
    private static var instance : BluetoothManager? = nil
    
    public static func sharedInstance() -> BluetoothManager {
        if instance == nil {
            let serialQueue = DispatchQueue(label: "ch.Hesso.nicolaswiedmer.iOS-ble-app.serialQueue")
            serialQueue.sync {
                instance = BluetoothManager();
            }
        }
        return instance!
    }
    
    func initCentralManager() {
        print("Init Central Manager")
        centralManager = CBCentralManager(delegate: self, queue: nil)
        
        psService = PowerSocketService()
        ctService = CurrentTimeService()
        tsService = TimeSettingsService()
        
        NotificationCenter.default.addObserver(self, selector: #selector(PowerSocketWriteCharValueNotification(notification:)), name: RCNotification.PowerSocketWriteCharacteristicValue, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(CurrentTimeWriteCharValueNotification(notification:)), name: RCNotification.CurrentTimeWriteCharacteristicValue, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(TimeSettingsWriteCharValueNotification(notification:)), name: RCNotification.TimeSettingsWriteCharacteristicValue, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(TimeSettingsWritePinCharValueNotification(notification:)), name: RCNotification.TimeSettingsWritePinCharacteristicValue, object: nil)
    }
    
    func centralManagerDidUpdateState(_ central: CBCentralManager) {
        switch central.state {
        case .poweredOff:
            break
        case .poweredOn:
            NotificationCenter.default.post(name: RCNotification.BluetoothReady, object: nil)
            break
        case .unknown:
            break
        case .resetting:
            break
        case .unsupported:
            break
        case .unauthorized:
            break
        }
    }
    
    func discoverDevice() {
        print("Starting Scan")
        NotificationCenter.default.post(name: RCNotification.StartScanning, object: nil)
        centralManager.scanForPeripherals(withServices: [PowerSocketUUID.SERVICE_PS], options: [CBCentralManagerScanOptionAllowDuplicatesKey:false] )
    }
    
    func centralManager(_ central: CBCentralManager, didDiscover peripheral: CBPeripheral, advertisementData: [String : Any], rssi RSSI: NSNumber) {
        peripherals.append(peripheral)
        let name : String
        if advertisementData["kCBAdvDataLocalName"] == nil {
            name = "N/A"
        } else {
            name = advertisementData["kCBAdvDataLocalName"] as! String
        }
        let address: String = peripheral.identifier.uuidString
        let rssi: String = RSSI.stringValue
        print("Device discovered: " + name + " " + address + " " + rssi)
        
        let description: [AnyHashable: Any] = [
            AnyHashable("deviceName") : name,
            AnyHashable("deviceAddress"): address,
            AnyHashable("deviceRSSI"): rssi,
        ]
        NotificationCenter.default.post(name: RCNotification.FoundDevice, object: nil, userInfo: description)
    }
    
    func getPeripherals() -> [CBPeripheral] {
        return peripherals
    }
    
    func stopScan() {
        NotificationCenter.default.post(name: RCNotification.StopScanning, object: nil)
        centralManager.stopScan()
    }
    
    func connectToDevice(device: CBPeripheral) {
        centralManager.connect(device, options: nil)
    }
    
    func centralManager(_ central: CBCentralManager, didConnect peripheral: CBPeripheral) {
        print("Connected with " + peripheral.name! + " " + peripheral.identifier.uuidString)
        //if peripheral.services == "22AEE6CF-C841-B1C0-A6BE-CC9D7DB44902" {
            bleDevice = peripheral
            bleDevice.delegate = self
            NotificationCenter.default.post(name: RCNotification.ConnectionComplete, object: nil)
        //}
    }
    
    func disconnectDevice() {
        centralManager.cancelPeripheralConnection(bleDevice)
        peripherals.removeAll()
        psService.disconnect()
        ctService.disconnect()
        tsService.disconnect()
    }
    
    func centralManager(_ central: CBCentralManager, didDisconnectPeripheral peripheral: CBPeripheral, error: Error?) {
        NotificationCenter.default.post(name: RCNotification.DisconnectedDevice, object: nil)
    }
    
    func discoverServices() {
        print("Start Discovering Services")
        bleDevice.discoverServices([PowerSocketUUID.SERVICE_PS, CurrentTimeServiceUUID.SERVICE_CT, TimeSettingsServiceUUID.SERVICE_TS])
    }
    
    func peripheral(_ peripheral: CBPeripheral, didDiscoverServices error: Error?) {
        print("didDiscoverServices")
        //psService = peripheral.services![0] as! PowerSocketService
        for serv in peripheral.services! {
            if serv.uuid == PowerSocketUUID.SERVICE_PS {
                psService.discoverService(service: serv)
                NotificationCenter.default.post(name: RCNotification.PowerSocketServiceScanComplete, object: nil)
            }
            if serv.uuid == CurrentTimeServiceUUID.SERVICE_CT {
                ctService.discoverService(service: serv)
                NotificationCenter.default.post(name: RCNotification.CurrentTimeServiceScanComplete, object: nil)
            }
            if serv.uuid == TimeSettingsServiceUUID.SERVICE_TS {
                tsService.discoverService(service: serv)
                NotificationCenter.default.post(name: RCNotification.TimeSettingsServiceScanComplete, object: nil)
            }
        }
    }
    
    func getPsService() -> PowerSocketService {
        return psService
    }
    
    func getCTService() -> CurrentTimeService {
        return ctService
    }
    
    func getTSService() -> TimeSettingsService {
        return tsService
    }
    
    func discoverPowerSocketCharacteristics() {
        bleDevice.discoverCharacteristics(nil, for: psService.getService())
    }
    
    func discoverCurrentTimeCharacteristics() {
        bleDevice.discoverCharacteristics(nil, for: ctService.getService())
    }
    
    func discoverTimeSettingsCharacteristics() {
        bleDevice.discoverCharacteristics(nil, for: tsService.getService())
    }
    
    func peripheral(_ peripheral: CBPeripheral, didDiscoverCharacteristicsFor service: CBService, error: Error?) {
        switch service {
        case psService.getService():
            psService.discoverCharacteristics()
            break
        case ctService.getService():
            ctService.discoverCharacteristic()
            break
        case tsService.getService():
            tsService.discoverCharacteristic()
        default:
            break
        }
    }
    
    @objc func PowerSocketWriteCharValueNotification(notification: NSNotification) {
        let not : [AnyHashable: Any] = notification.userInfo!
        let value : [UInt8] = not[AnyHashable("value")] as! [UInt8]
        let na = Data(bytes: value)
        bleDevice.writeValue(na, for: psService.getWriteCharacteristic(), type: CBCharacteristicWriteType.withResponse)
    }
    
    @objc func CurrentTimeWriteCharValueNotification(notification: NSNotification) {
        let not : [AnyHashable: Any] = notification.userInfo!
        let value : [UInt8] = not[AnyHashable("time")] as! [UInt8]
        let na = Data(bytes: value)
        print(na)
        bleDevice.writeValue(na, for: ctService.getWriteCharacteristic(), type: CBCharacteristicWriteType.withResponse)
    }
    
    @objc func TimeSettingsWriteCharValueNotification(notification: NSNotification) {
        let not : [AnyHashable: Any] = notification.userInfo!
        let value : [UInt8] = not[AnyHashable("settings")] as! [UInt8]
        let na = Data(bytes: value)
        //print(na)
        bleDevice.writeValue(na, for: tsService.getWriteTimeCharacteristic(), type: CBCharacteristicWriteType.withResponse)
        NotificationCenter.default.post(name: RCNotification.TimeSettingsWriteCharFinished, object: nil)
    }
    
    @objc func TimeSettingsWritePinCharValueNotification(notification: NSNotification) {
        let not : [AnyHashable: Any] = notification.userInfo!
        let pinValues : [UInt8] = not[AnyHashable("timePins")] as! [UInt8]
        let pi = Data(bytes: pinValues)
        bleDevice.writeValue(pi, for: tsService.getWriteTimePinCharacteristic(), type: CBCharacteristicWriteType.withResponse)
    }
    
    
    
    func readPowerSocketCharacteristic() {
        bleDevice.readValue(for: psService.getReadCharacteristic())
        bleDevice.setNotifyValue(true, for: psService.getReadCharacteristic())
    }
    
    func peripheral(_ peripheral: CBPeripheral, didUpdateValueFor characteristic: CBCharacteristic, error: Error?) {
        print(characteristic.value!)
        if(characteristic.uuid == PowerSocketUUID.CHAR_PS_PIN_READ) {
            for i: Int in 0..<14 {
                let valuessss : UInt8 = characteristic.value?[i] ?? 0
                print("ReadValues:")
                print(valuessss)
                print("-----------------------")
            }
            
            let readVal: [AnyHashable: Any] = [
                AnyHashable("readValue"): characteristic.value!
            ]
            
            NotificationCenter.default.post(name: RCNotification.PowerSocketReadCharacteristicValue, object: nil, userInfo: readVal)
        } else {
            print("Other characteristic did updated")
        }
        
    }
    
}
